/* ============================================================================
 * Copyright (C) 2004 - Infineon Technologies AG.
 *
 * All rights reserved.
 * ============================================================================
 *
 * ============================================================================
 *
 * This document contains proprietary information belonging to Infineon
 * Technologies AG. Passing on and copying of this document, and communication
 * of its contents is not permitted without prior written authorisation.
 *
 * ============================================================================
 */

/* ===========================================================================
 *
 * File Name    : adslAturChanTable_access.c
 * Author       : Subbi
 * Date         : 17-Jun-2004
 *
 * ===========================================================================
 *
 * Project      : Amazon
 * Block        : ADSL-MIBs
 *
 * ===========================================================================
 * Contents: This file contains functions for:
 *                 1) Getting the data context pointer & loop context pointer
 *                 2) User-defined data access functions (per column) for data
 *                    in table adslAturChanTable
 *
 *
 * ===========================================================================
 * References: RFC 2662
 *
 */

/* ===========================================================================
 * Revision History:
 *
 * $Log$
 * ===========================================================================
 */

/*
 * Note: this file originally auto-generated by mib2c using
 *        : mib2c.access_functions.conf,v 1.6 2003/09/11 22:48:49 rstory Exp $
 */

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
 */
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <net-snmp/agent/net-snmp-agent-includes.h>
#include "adslAturChanTable_access.h"
#ifdef STUB_IOCTL
#include "adslAturChanTable_enums.h"
#endif

/* column number definitions for table adslAturChanTable */
#include "adslAturChanTable_columns.h"

/*
 * ===========================================================================
 *                           GLOBAL DEFINITIONS
 * ===========================================================================
 */
// *** Global variable definitions
static long long_ret;

/*
 * ===========================================================================
 *                           LOCAL FUNCTIONS
 * ===========================================================================
 */
/** returns the first data point within the adslAturChanTable table data.

    Set the my_loop_context variable to the first data point structure
    of your choice (from which you can find the next one).  This could
    be anything from the first node in a linked list, to an integer
    pointer containing the beginning of an array variable.

    Set the my_data_context variable to something to be returned to
    you later that will provide you with the data to return in a given
    row.  This could be the same pointer as what my_loop_context is
    set to, or something different.

    The put_index_data variable contains a list of snmp variable
    bindings, one for each index in your table.  Set the values of
    each appropriately according to the data matching the first row
    and return the put_index_data variable at the end of the function.
*/
netsnmp_variable_list *
adslAturChanTable_get_first_data_point(void **my_loop_context,
                                       void **my_data_context,
                                       netsnmp_variable_list *
                                       put_index_data,
                                       netsnmp_iterator_info *mydata)
{

   int *My_Data_Context;
   My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
   if (!My_Data_Context) {
       put_index_data = NULL;
       snmp_log(LOG_ERR, "Get-First: adslAturChanTable: My_Data_Context malloc failed\n");
       return put_index_data;
   }
   *My_Data_Context=IFTYPE_ADSL_FAST_INTERLEAVE;
   *my_data_context=My_Data_Context;

   put_index_data = adslTable_get_first_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}

/** functionally the same as adslAturChanTable_get_first_data_point, but
   my_loop_context has already been set to a previous value and should
   be updated to the next in the list.  For example, if it was a
   linked list, you might want to cast it to your local data type and
   then return my_loop_context->next.  The my_data_context pointer
   should be set to something you need later and the indexes in
   put_index_data updated again. */
netsnmp_variable_list *
adslAturChanTable_get_next_data_point(void **my_loop_context,
                                      void **my_data_context,
                                      netsnmp_variable_list *
                                      put_index_data,
                                      netsnmp_iterator_info *mydata)
{

    int *My_Data_Context;
    My_Data_Context = SNMP_MALLOC_TYPEDEF(int);
    if (!My_Data_Context) {
       put_index_data = NULL;
       snmp_log(LOG_ERR, "Get-Next: adslAturChanTable: My_Data_Context malloc failed\n");
       return put_index_data;
    }
    *My_Data_Context=IFTYPE_ADSL_FAST_INTERLEAVE;
    *my_data_context=My_Data_Context;

    put_index_data = adslTable_get_next_data_point(my_loop_context,
                                   my_data_context,
                                   put_index_data,
                                   mydata);

    SNMP_FREE(My_Data_Context);
    return put_index_data;
}


/*
 *  User-defined data access functions (per column) for data in table
    adslAturChanTable */
/*
 * NOTE:
 * - these get_ routines MUST return data that will not be freed (ie,
 *   use static variables or persistent data).  It will be copied, if
 *   needed, immediately after the get_ routine has been called.
 * - these SET routines must copy the incoming data and can not take
 *   ownership of the memory passed in by the val pointer.
 */


#if defined(HAVE_OPTIMIZED_CODE)

u_int32         *
get_adslAturChanInfo(void *data_context, size_t * ret_len,
		unsigned int var)
{
  unsigned int flags = 0;	
  struct adslAturChanInfo adslAturChanStruct;
  memset(&adslAturChanStruct,0,sizeof(adslAturChanInfo));
  adslAturChanStruct.ifIndex= (*(int *)data_context);

  switch (var)  {

        case COLUMN_ADSLATURCHANINTERLEAVEDELAY:
	   flags = ATUR_CHAN_INTLV_DELAY_FLAG;
	   break;

        case COLUMN_ADSLATURCHANCURRTXRATE:
	   flags = ATUR_CHAN_CURR_TX_RATE_FLAG;
	   break;

        case COLUMN_ADSLATURCHANPREVTXRATE:
	   flags = ATUR_CHAN_PREV_TX_RATE_FLAG;
	   break;

        case COLUMN_ADSLATURCHANCRCBLOCKLENGTH:
	   flags = ATUR_CHAN_CRC_BLK_LEN_FLAG;
	   break;

     	default:
     	   DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
     	   return NULL;
  }

  SET_FLAG(&adslAturChanStruct.flags, flags);
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_CHAN_INFO, &adslAturChanStruct);
#else
    get_adslAturChanTable(Fd, GET_ADSL_ATUR_CHAN_INFO,
        (void *) &adslAturChanStruct);
#endif
    if (IS_FLAG_SET(&adslAturChanStruct.flags, flags) == 0)
       DEBUGMSGTL(("adslMIB-ATUR", "ChanTable: GET- %x CMV fail\n", flags));

    switch (flags)  {
	    
	case ATUR_CHAN_INTLV_DELAY_FLAG:
    	   long_ret = adslAturChanStruct.interleaveDelay;
	   break;

	case ATUR_CHAN_CURR_TX_RATE_FLAG:
     	   long_ret = adslAturChanStruct.currTxRate;
	   break;

	case ATUR_CHAN_PREV_TX_RATE_FLAG:
    	   long_ret = adslAturChanStruct.prevTxRate;
	   break;

	case ATUR_CHAN_CRC_BLK_LEN_FLAG:
    	   long_ret = adslAturChanStruct.crcBlkLen;
	   break;

        default:
     	    DEBUGMSGTL(("Unknown MIB variable requested!", "\n"));
            return NULL;
    }
    
    CLR_FLAG(&adslAturChanStruct.flags, flags);
    *ret_len = sizeof(u_int32);
    return (u_int32 *) &long_ret;
}

#else

/** XXX: return a data pointer to the data for the adslAturChanInterleaveDelay
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAturChanInterleaveDelay(void *data_context, size_t * ret_len)
{
    struct adslAturChanInfo adslAturChanStruct;

    memset(&adslAturChanStruct,0,sizeof(adslAturChanInfo));
    adslAturChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_INTLV_DELAY_FLAG);
    
#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_CHAN_INFO, &adslAturChanStruct);
#else
    get_adslAturChanTable(Fd, GET_ADSL_ATUR_CHAN_INFO,
        (void *) &adslAturChanStruct);
#endif
    if (IS_FLAG_SET(&adslAturChanStruct.flags, ATUR_CHAN_INTLV_DELAY_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUR", "ChanIntlvDelay CMV fail\n"));

    CLR_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_INTLV_DELAY_FLAG);
    long_ret = adslAturChanStruct.interleaveDelay;
    *ret_len = sizeof(adslAturChanStruct.interleaveDelay);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAturChanCurrTxRate column
    and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAturChanCurrTxRate(void *data_context, size_t * ret_len)
{
    struct adslAturChanInfo adslAturChanStruct;

    memset(&adslAturChanStruct,0,sizeof(adslAturChanInfo));
    adslAturChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_CURR_TX_RATE_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_CHAN_INFO, &adslAturChanStruct);
#else
    get_adslAturChanTable(Fd, GET_ADSL_ATUR_CHAN_INFO,
        (void *) &adslAturChanStruct);
#endif
    if (IS_FLAG_SET(&adslAturChanStruct.flags, ATUR_CHAN_CURR_TX_RATE_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUR", "ChanCurrTxRate CMV fail\n"));

    CLR_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_CURR_TX_RATE_FLAG);
    long_ret = adslAturChanStruct.currTxRate;
    *ret_len = sizeof(adslAturChanStruct.currTxRate);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAturChanPrevTxRate column
    and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAturChanPrevTxRate(void *data_context, size_t * ret_len)
{
    struct adslAturChanInfo adslAturChanStruct;

    memset(&adslAturChanStruct,0,sizeof(adslAturChanInfo));
    adslAturChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_PREV_TX_RATE_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_CHAN_INFO, &adslAturChanStruct);
#else
    get_adslAturChanTable(Fd, GET_ADSL_ATUR_CHAN_INFO,
        (void *) &adslAturChanStruct);
#endif
    if (IS_FLAG_SET(&adslAturChanStruct.flags, ATUR_CHAN_PREV_TX_RATE_FLAG) == 0) 
        DEBUGMSGTL(("adslMIB-ATUR", "ChanPrevTxRate CMV fail\n"));

    CLR_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_PREV_TX_RATE_FLAG);
    long_ret = adslAturChanStruct.prevTxRate;
    *ret_len = sizeof(adslAturChanStruct.prevTxRate);
    return (u_int32 *) &long_ret;
}

/** XXX: return a data pointer to the data for the adslAturChanCrcBlockLength
    column and set ret_len to its proper size in bytes. */
u_int32         *
get_adslAturChanCrcBlockLength(void *data_context, size_t * ret_len)
{
    struct adslAturChanInfo adslAturChanStruct;

    memset(&adslAturChanStruct,0,sizeof(adslAturChanInfo));
    adslAturChanStruct.ifIndex= (*(int *)data_context);
    SET_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_CRC_BLK_LEN_FLAG);

#ifdef ENABLE_DRIVER_CALL
    adsl_ioctl(Fd, GET_ADSL_ATUR_CHAN_INFO, &adslAturChanStruct);
#else
    get_adslAturChanTable(Fd, GET_ADSL_ATUR_CHAN_INFO,
        (void *) &adslAturChanStruct);
#endif
    if (IS_FLAG_SET(&adslAturChanStruct.flags, ATUR_CHAN_CRC_BLK_LEN_FLAG) == 0) 
       DEBUGMSGTL(("adslMIB-ATUR", "ChanCRCBlkLen CMV fail\n"));

    CLR_FLAG(&adslAturChanStruct.flags, ATUR_CHAN_CRC_BLK_LEN_FLAG);
    long_ret = adslAturChanStruct.crcBlkLen;
    *ret_len = sizeof(adslAturChanStruct.crcBlkLen);
    return (u_int32 *) &long_ret;
}
#endif


#ifndef ENABLE_DRIVER_CALL
int  get_adslAturChanTable(int Fd2,int GET_ADSL_atur_CHAN_INFO,
            void *adslAturChanTable)
{
    adslAturChanInfo *adslAturChanValue;
    adslAturChanValue=(adslAturChanInfo *)adslAturChanTable;
    adslAturChanValue->interleaveDelay = 40;
    adslAturChanValue->currTxRate = 40;
    adslAturChanValue->prevTxRate = 40;
    adslAturChanValue->crcBlkLen = 10;
    return 0;
}
#endif

