/************************************************************************
 *
 *    Copyright (c) 2005
 *    Infineon Technologies AG
 *    St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 *    This document contains proprietary information belonging to Infineon
 *    Technologies AG. Passing on and copying of this document, and communication
 *    of its contents is not permitted without prior written authorisation.
 *
 *    --------------------------------------------------------------------
 *    
 *        Project : INCA-IP
 *        Block   : IFX MIB
 *        Creator : Sharon Tseng
 *        File    : ifxMIB_access.h
 *        Abstract: 
 *                  
 *        Date    : 18-Jul-2005
 *        Ref. Doc: IIP-DFS.doc
 *    
 *    Modification History:
 *           By              Date     Ver.   Modification Description
 *           --------------- -------- -----  -----------------------------
 *
 ************************************************************************/

#ifndef _IFXCOMMONMIB_H_
#define _IFXCOMMONMIB_H_

/*
 * ===========================================================================
 *                           INCLUDE FILES
 * ===========================================================================
*/
#include "ifx_emf.h"
#include "ifx_common_defs.h"
#include "ifx_ipc.h"
#include "ifx_cm.h"

/*
 * ===========================================================================
 *                          DEFINITIONS
 * ===========================================================================
 */
#define IIP_MEM_LEAK_FIX

/*
 * enums for row status
 */
#define ROWSTATUS_ACTIVE                        1
#define ROWSTATUS_NOTINSERVICE                  2
#define ROWSTATUS_NOTREADY                      3
#define ROWSTATUS_CREATEANDGO                   4
#define ROWSTATUS_CREATEANDWAIT                 5
#define ROWSTATUS_DESTROY                       6

/* enums for column rtpSessionMonitor */
#define RTPSESSIONMONITOR_TRUE                  1
#define RTPSESSIONMONITOR_FALSE                 2

#define IFX_INDEX_TYPE_INT                      1
#define IFX_INDEX_TYPE_CHAR                     2
#define FILE_RC_CONF                            "/etc/rc.conf"
#define MAX_IP_ADDRESS                          16 /* 100.100.100.100 */
#define MAX_MAC_ADDRESS                         18 /* FF-FF-FF-FF-FF-FF */
#define IFX_MIB                                 15789
#define IFX_PRODUCT                             1  /* 1 : IIP
                                                      2 : ATA
                                                      3 : Amazon */

/* Interface Type --> from IANAifType-MIB.txt file */
#define IFTYPE_ETHERNET_CSMACD                  6   /* ethernetCsmacd(6) */
#define IFTYPE_VOICEEM							100 /* voiceEM(100) */
#define IFTYPE_VOICEFXO							101 /* voiceFXO(101) */
#define IFTYPE_VOICEFXS							102 /* voiceFXS(102) */
#define IFTYPE_VOICEENCAP						103 /* voiceEncap(103) */
#define IFTYPE_VOICEOVERIP						104 /* voiceOverIp(104) */

#define IFX_WEB_MAX_PB_ENTRIES                  100
#define BUF_SIZE_50K							10240
#define BUF_SIZE_1K                             1024
#define MAX_DATA_LEN                            128
#define MAX_FILELINE_LEN						256
#define MAX_FILENAME_LEN						256
#define MAX_TAG_LEN                             64

/*
 * ===========================================================================
 *                           STRUCTURE / ENUM TYPE
 * ===========================================================================
*/
typedef enum
{
    IFX_MIB_WAN_DNS=3,
    IFX_MIB_WAN_PPPOE,
    IFX_MIB_WAN_STATIC,
    IFX_MIB_QOS_EXTEND,
    IFX_MIB_QOS_SWENHANCE,
    IFX_MIB_QOS_STATICMAC,
    IFX_MIB_VOIP_CODEC

} e_IFX_MIB_Enum;

typedef struct
{
    unsigned int     nType;     /* e_IFX_MIB_Enum type of structure */
    unsigned int     nCount;    /* number of rows in structure */
    unsigned int     nSize;     /* structure's size */
    unsigned int     nPSize;    /* structure's pointer size */

} IFX_MIB_MSG_S;

typedef struct
{
	unsigned int	value;      /* value index */
    char            str[4];     /* value index content */
} IFX_MIB_ENUM_S;

/* RTP Session Table */
typedef struct
{
    unsigned int       rtpSessionIndex;
    long               rtpSessionDomain;
    unsigned char      rtpSessionRemAddr[MAX_IP_ADDRESS];
    unsigned char      rtpSessionLocAddr[MAX_IP_ADDRESS];
    long               rtpSessionIfIndex;
    unsigned int       rtpSessionSenderJoins;
    unsigned int       rtpSessionReceiverJoins;
    unsigned int       rtpSessionByes;
    unsigned int       rtpSessionStartTime;
    long               rtpSessionMonitor;
    long               rtpSessionRowStatus;

} RTPSESSIONENTRY_S;

/* RTP Recver Table */
typedef struct
{
    int           index;
    uint32        uiRcvrSRCSSRC;
    uint32        uiRcvrSSRC;
    char          rtpRcvrCNAME[IFX_CM_RTP_MIB_TEXT_LEN];
    unsigned char rtpRcvrAddr[MAX_IP_ADDRESS];
    unsigned int  rtpRcvrRTT;
    uint64        rtpRcvrLostPackets;
    unsigned int  rtpRcvrJitter;
    char          rtpRcvrTool[IFX_CM_RTP_MIB_TEXT_LEN];
    unsigned int  rtpRcvrRRs;
    unsigned int  rtpRcvrRRTime;
    int           rtpRcvrPT;
    uint64        rtpRcvrPackets;
    uint64        rtpRcvrOctets;
    unsigned int  rtpRcvrStartTime;

} RTPRXENTRY_S;

/* RTP Sender Table */
/** Index rtpSessionIndex is external */
/** Index rtpSenderSSRC is internal */
typedef struct
{
    int           index;
    uint32        uiSenderSSRC;
    char          rtpSenderCNAME[IFX_CM_RTP_MIB_TEXT_LEN];
    unsigned char rtpSenderAddr[MAX_IP_ADDRESS];
    uint64        rtpSenderPackets;
    uint64        rtpSenderOctets;
    char          rtpSenderTool[IFX_CM_RTP_MIB_TEXT_LEN];
    unsigned int  rtpSenderSRs;
    unsigned int  rtpSenderSRTime;
    int           rtpSenderPT;
    unsigned int  rtpSenderStartTime;

} RTPTXENTRY_S;

/* DNS Configuration*/
typedef struct
{
    int index;
    char cPriServerMAC[MAX_IP_ADDRESS];
    char cSecServerMAC[MAX_IP_ADDRESS];

} DNSCFG_S;

/* PPPoE Configuration*/
typedef struct
{
    int index;
    char cPppoeUserName[60];
    char cPppoePasswd[60];
    char cPppoeISP[60];
    u_long ulPppoeAuthType;

} PPPOECFG_S;

/* LAN Static Configuration*/
typedef struct
{
    int index;
    char cIpAddr[MAX_IP_ADDRESS];
    char cNetmask[MAX_IP_ADDRESS];
    char cGateway[MAX_IP_ADDRESS];

} STATICCFG_S;

/* QoS Stream Classification Configuration*/
typedef struct
{
    int index;
    int nMgtPrio;
    int nMgtId;
    char cMgtData[3];
    int nVoipPrio;
    int nVoipId;
    char cVoipData[3];
    int nSignalPrio;
    int nSignalId;
    char cSignalData[3];
    int nOtherPrio;
    int nOtherId;
    char cOtherData[3];

} QOSEXTENDCFG_S;

/* VLAN Switch Enhance Configuration*/
typedef struct
{
    int index;
    int nVID;
    int nAccLAN;
    int nAccPC;
    int nAccIPP;
    int nEgrLAN;
    int nEgrPC;
    int nEgrIPP;

} QOSSWENHANCECFG_S;

/* VLAN Static MAC Configuration*/
typedef struct
{
    int index;
    char cMacAddress[MAX_MAC_ADDRESS];
    int nPortType;
    int nCriticalTraffic;

} QOSSTATICMACCFG_S;

/* VoIP Codec Configuration*/
typedef struct
{
    u_long ulEndPointIdx;
    char cUserName[IFX_CM_MAX_USR_NAME];
    char cDomainName[IFX_CM_MAX_DOMAIN_NAME];
    u_long ulIndex;
    u_long ulMediaBandwidth;
    u_long ulCodecName;
    u_long ulPayloadType;
    u_long ulPriority;
    u_long ulFrameSize;
    long lRowStatus;

} VOIPCODECCFG_S;

/*
 * ===========================================================================
 *                           EXTERN VARIABLES
 * ===========================================================================
*/
extern void                 **g_void;
extern IFX_MIB_MSG_S        xMibMsg;
extern IFX_MIB_MSG_S        xPrevMibMsg;

extern RTPSESSIONENTRY_S    **pstRtpSessCfg;
extern RTPTXENTRY_S         **pstRtpTxCfg;
extern RTPRXENTRY_S         **pstRtpRxCfg;
extern DNSCFG_S             **pstWanDnsCfg;
extern PPPOECFG_S           **pstWanPppoeCfg;
extern STATICCFG_S          **pstWanStaticCfg;
extern QOSEXTENDCFG_S       **pstQosExtendCfg;
extern QOSSWENHANCECFG_S    **pstQosSwEnCfg;
extern QOSSTATICMACCFG_S    **pstQosStaticMacCfg;
extern VOIPCODECCFG_S       **pstVoipCodec;

#endif /* _IFXCOMMONMIB_H_ */
